@@ -73,7 +73,6 @@ typedef struct {
 	GPtrArray *auth_types;
 	gboolean auto_ntlm;
 
-	GMutex lock;
 	SoupAuth *proxy_auth;
 	GHashTable *auth_hosts;
 } SoupAuthManagerPrivate;
@@ -104,7 +103,6 @@ soup_auth_manager_init (SoupAuthManager *manager)
 						  soup_uri_host_equal,
 						  NULL,
 						  (GDestroyNotify)soup_auth_host_free);
-	g_mutex_init (&priv->lock);
 }
 
 static void
@@ -118,8 +116,6 @@ soup_auth_manager_finalize (GObject *object)
 
 	g_clear_object (&priv->proxy_auth);
 
-	g_mutex_clear (&priv->lock);
-
 	G_OBJECT_CLASS (soup_auth_manager_parent_class)->finalize (object);
 }
 
@@ -638,8 +634,6 @@ auth_got_headers (SoupMessage *msg, gpointer manager)
 	SoupAuth *auth, *prior_auth;
 	gboolean prior_auth_failed = FALSE;
 
-	g_mutex_lock (&priv->lock);
-
 	/* See if we used auth last time */
 	prior_auth = soup_message_get_auth (msg);
 	if (prior_auth && check_auth (msg, prior_auth)) {
@@ -648,10 +642,8 @@ auth_got_headers (SoupMessage *msg, gpointer manager)
 			prior_auth_failed = TRUE;
 	} else {
 		auth = create_auth (priv, msg);
-		if (!auth) {
-			g_mutex_unlock (&priv->lock);
+		if (!auth)
 			return;
-		}
 	}
 
 	if (!soup_message_query_flags (msg, SOUP_MESSAGE_DO_NOT_USE_AUTH_CACHE)) {
@@ -668,7 +660,6 @@ auth_got_headers (SoupMessage *msg, gpointer manager)
 			   prior_auth_failed, FALSE, TRUE);
 	soup_message_set_auth (msg, auth);
 	g_object_unref (auth);
-	g_mutex_unlock (&priv->lock);
 }
 
 static void
@@ -677,7 +668,6 @@ auth_got_body (SoupMessage *msg, gpointer manager)
         SoupAuthManagerPrivate *priv = soup_auth_manager_get_instance_private (manager);
 	SoupAuth *auth;
 
-	g_mutex_lock (&priv->lock);
 	auth = lookup_auth (priv, msg);
 	if (auth && soup_auth_is_ready (auth, msg)) {
 		if (SOUP_IS_CONNECTION_AUTH (auth))
@@ -691,7 +681,6 @@ auth_got_body (SoupMessage *msg, gpointer manager)
 
 		soup_session_requeue_message (priv->session, msg);
 	}
-	g_mutex_unlock (&priv->lock);
 }
 
 static void
@@ -701,8 +690,6 @@ proxy_auth_got_headers (SoupMessage *msg, gpointer manager)
 	SoupAuth *auth = NULL, *prior_auth;
 	gboolean prior_auth_failed = FALSE;
 
-	g_mutex_lock (&priv->lock);
-
 	/* See if we used auth last time */
 	prior_auth = soup_message_get_proxy_auth (msg);
 	if (prior_auth && check_auth (msg, prior_auth)) {
@@ -715,10 +702,9 @@ proxy_auth_got_headers (SoupMessage *msg, gpointer manager)
 
 	if (!auth) {
 		auth = create_auth (priv, msg);
-		if (!auth) {
-			g_mutex_unlock (&priv->lock);
+		if (!auth)
 			return;
-		}
+
 		if (!soup_message_query_flags (msg, SOUP_MESSAGE_DO_NOT_USE_AUTH_CACHE))
 			priv->proxy_auth = g_object_ref (auth);
 	}
@@ -728,7 +714,6 @@ proxy_auth_got_headers (SoupMessage *msg, gpointer manager)
 			   prior_auth_failed, TRUE, TRUE);
 	soup_message_set_proxy_auth (msg, auth);
 	g_object_unref (auth);
-	g_mutex_unlock (&priv->lock);
 }
 
 static void
@@ -737,8 +722,6 @@ proxy_auth_got_body (SoupMessage *msg, gpointer manager)
         SoupAuthManagerPrivate *priv = soup_auth_manager_get_instance_private (manager);
 	SoupAuth *auth;
 
-	g_mutex_lock (&priv->lock);
-
 	auth = lookup_proxy_auth (priv, msg);
 	if (auth && soup_auth_is_ready (auth, msg)) {
 		/* When not using cached credentials, update the Authorization header
@@ -748,8 +731,6 @@ proxy_auth_got_body (SoupMessage *msg, gpointer manager)
 			update_authorization_header (msg, auth, TRUE);
 		soup_session_requeue_message (priv->session, msg);
 	}
-
-	g_mutex_unlock (&priv->lock);
 }
 
 static void
@@ -761,8 +742,6 @@ auth_msg_starting (SoupMessage *msg, gpointer manager)
 	if (soup_message_query_flags (msg, SOUP_MESSAGE_DO_NOT_USE_AUTH_CACHE))
 		return;
 
-	g_mutex_lock (&priv->lock);
-
 	if (soup_message_get_method (msg) != SOUP_METHOD_CONNECT) {
 		auth = lookup_auth (priv, msg);
 		if (auth) {
@@ -782,8 +761,6 @@ auth_msg_starting (SoupMessage *msg, gpointer manager)
 	}
 	soup_message_set_proxy_auth (msg, auth);
 	update_authorization_header (msg, auth, TRUE);
-
-	g_mutex_unlock (&priv->lock);
 }
 
 static void
@@ -840,9 +817,7 @@ soup_auth_manager_use_auth (SoupAuthManager *manager,
 {
         SoupAuthManagerPrivate *priv = soup_auth_manager_get_instance_private (manager);
 
-	g_mutex_lock (&priv->lock);
 	record_auth_for_uri (priv, uri, auth, FALSE);
-	g_mutex_unlock (&priv->lock);
 }
 
 /**
@@ -859,9 +834,7 @@ soup_auth_manager_clear_cached_credentials (SoupAuthManager *manager)
 
 	g_return_if_fail (SOUP_IS_AUTH_MANAGER (manager));
 
-	g_mutex_lock (&priv->lock);
 	g_hash_table_remove_all (priv->auth_hosts);
-	g_mutex_unlock (&priv->lock);
 }
 
 static void
