@@ -111,6 +111,23 @@ soup_client_input_stream_read_fn (GInputStream  *stream,
 	return nread;
 }
 
+static gssize
+soup_client_input_stream_skip (GInputStream  *stream,
+                               gsize          count,
+                               GCancellable  *cancellable,
+                               GError       **error)
+{
+        gssize nread;
+
+        nread = G_INPUT_STREAM_CLASS (soup_client_input_stream_parent_class)->
+                skip (stream, count, cancellable, error);
+
+        if (nread == 0)
+                g_signal_emit (stream, signals[SIGNAL_EOF], 0);
+
+        return nread;
+}
+
 static gssize
 soup_client_input_stream_read_nonblocking (GPollableInputStream  *stream,
 					   void                  *buffer,
@@ -233,6 +250,7 @@ soup_client_input_stream_class_init (SoupClientInputStreamClass *stream_class)
 	object_class->get_property = soup_client_input_stream_get_property;
 
 	input_stream_class->read_fn = soup_client_input_stream_read_fn;
+	input_stream_class->skip = soup_client_input_stream_skip;
 	input_stream_class->close_fn = soup_client_input_stream_close_fn;
 	input_stream_class->close_async = soup_client_input_stream_close_async;
 	input_stream_class->close_finish = soup_client_input_stream_close_finish;
