@@ -16,13 +16,12 @@
 
 /**
  * SECTION:soup-form
+ * @section_id: SoupForm
  * @short_description: HTML form handling
  * @see_also: #SoupMultipart
  *
  * libsoup contains several help methods for processing HTML forms as
- * defined by <ulink
- * url="http://www.w3.org/TR/html401/interact/forms.html#h-17.13">the
- * HTML 4.01 specification</ulink>.
+ * defined by the [the HTML 4.01 specification](http://www.w3.org/TR/html401/interact/forms.html#h-17.13).
  **/
 
 /**
@@ -32,7 +31,6 @@
  * <literal>"application/x-www-form-urlencoded"</literal>; the default
  * MIME type for POSTing HTML form data.
  *
- * Since: 2.26
  **/
 
 /**
@@ -42,7 +40,6 @@
  * <literal>"multipart/form-data"</literal>; the MIME type used for
  * posting form data that contains files to be uploaded.
  *
- * Since: 2.26
  **/
 
 #define XDIGIT(c) ((c) <= '9' ? (c) - '0' : ((c) & 0x4F) - 'A' + 10)
@@ -77,7 +74,7 @@ form_decode (char *part)
  * Decodes @form, which is an urlencoded dataset as defined in the
  * HTML 4.01 spec.
  *
- * Return value: (element-type utf8 utf8) (transfer full): a hash
+ * Returns: (element-type utf8 utf8) (transfer container): a hash
  * table containing the name/value pairs from @encoded_form, which you
  * can free with g_hash_table_destroy().
  **/
@@ -113,13 +110,13 @@ soup_form_decode (const char *encoded_form)
 
 /**
  * soup_form_decode_multipart:
- * @msg: a #SoupMessage containing a "multipart/form-data" request body
+ * @multipart: a #SoupMultipart
  * @file_control_name: (allow-none): the name of the HTML file upload control, or %NULL
  * @filename: (out) (allow-none): return location for the name of the uploaded file, or %NULL
  * @content_type: (out) (allow-none): return location for the MIME type of the uploaded file, or %NULL
  * @file: (out) (allow-none): return location for the uploaded file data, or %NULL
  *
- * Decodes the "multipart/form-data" request in @msg; this is a
+ * Decodes the "multipart/form-data" request in @multipart; this is a
  * convenience method for the case when you have a single file upload
  * control in a form. (Or when you don't have any file upload
  * controls, but are still using "multipart/form-data" anyway.) Pass
@@ -133,37 +130,31 @@ soup_form_decode (const char *encoded_form)
  * care about those fields. soup_form_decode_multipart() may also
  * return %NULL in those fields if the client did not provide that
  * information. You must free the returned filename and content-type
- * with g_free(), and the returned file data with soup_buffer_free().
+ * with g_free(), and the returned file data with g_bytes_unref().
  *
  * If you have a form with more than one file upload control, you will
  * need to decode it manually, using soup_multipart_new_from_message()
  * and soup_multipart_get_part().
  *
- * Return value: (nullable) (element-type utf8 utf8) (transfer full):
+ * Returns: (nullable) (element-type utf8 utf8) (transfer container):
  * a hash table containing the name/value pairs (other than
  * @file_control_name) from @msg, which you can free with
  * g_hash_table_destroy(). On error, it will return %NULL.
- *
- * Since: 2.26
- **/
+ */
 GHashTable *
-soup_form_decode_multipart (SoupMessage *msg, const char *file_control_name,
-			    char **filename, char **content_type,
-			    SoupBuffer **file)
+soup_form_decode_multipart (SoupMultipart *multipart,
+			    const char    *file_control_name,
+			    char         **filename,
+			    char         **content_type,
+			    GBytes       **file)
 {
-	SoupMultipart *multipart;
 	GHashTable *form_data_set, *params;
 	SoupMessageHeaders *part_headers;
-	SoupBuffer *part_body;
+	GBytes *part_body;
 	char *disposition, *name;
 	int i;
 
-	g_return_val_if_fail (SOUP_IS_MESSAGE (msg), NULL);
-
-	multipart = soup_multipart_new_from_message (msg->request_headers,
-						     msg->request_body);
-	if (!multipart)
-		return NULL;
+	g_return_val_if_fail (multipart != NULL, NULL);
 
 	if (filename)
 		*filename = NULL;
@@ -193,12 +184,12 @@ soup_form_decode_multipart (SoupMessage *msg, const char *file_control_name,
 			if (content_type)
 				*content_type = g_strdup (soup_message_headers_get_content_type (part_headers, NULL));
 			if (file)
-				*file = soup_buffer_copy (part_body);
+				*file = g_bytes_ref (part_body);
 		} else {
 			g_hash_table_insert (form_data_set,
 					     g_strdup (name),
-					     g_strndup (part_body->data,
-							part_body->length));
+					     g_strndup (g_bytes_get_data (part_body, NULL),
+							g_bytes_get_size (part_body)));
 		}
 
 		g_free (disposition);
@@ -254,7 +245,9 @@ encode_pair (GString *str, const char *name, const char *value)
  * working with dynamic forms, use soup_form_encode_hash() or
  * soup_form_encode_datalist().
  *
- * Return value: the encoded form
+ * Returns: the encoded form
+ *
+ * See also: soup_message_new_from_encoded_form()
  **/
 char *
 soup_form_encode (const char *first_field, ...)
@@ -283,7 +276,9 @@ soup_form_encode (const char *first_field, ...)
  * takes a hash table, it cannot enforce that; if you care about the
  * ordering of the form fields, use soup_form_encode_datalist().
  *
- * Return value: the encoded form
+ * Returns: the encoded form
+ *
+ * See also: soup_message_new_from_encoded_form()
  **/
 char *
 soup_form_encode_hash (GHashTable *form_data_set)
@@ -313,7 +308,9 @@ datalist_encode_foreach (GQuark key_id, gpointer value, gpointer str)
  * spec. Unlike soup_form_encode_hash(), this preserves the ordering
  * of the form elements, which may be required in some situations.
  *
- * Return value: the encoded form
+ * Returns: the encoded form
+ *
+ * See also: soup_message_new_from_encoded_form()
  **/
 char *
 soup_form_encode_datalist (GData **form_data_set)
@@ -330,10 +327,11 @@ soup_form_encode_datalist (GData **form_data_set)
  * @args: pointer to additional values, as in soup_form_encode()
  *
  * See soup_form_encode(). This is mostly an internal method, used by
- * various other methods such as soup_uri_set_query_from_fields() and
- * soup_form_request_new().
+ * various other methods such as soup_form_encode().
  *
- * Return value: the encoded form
+ * Returns: the encoded form
+ *
+ * See also: soup_message_new_from_encoded_form()
  **/
 char *
 soup_form_encode_valist (const char *first_field, va_list args)
@@ -353,139 +351,3 @@ soup_form_encode_valist (const char *first_field, va_list args)
 
 	return g_string_free (str, FALSE);
 }
-
-static SoupMessage *
-soup_form_request_for_data (const char *method, const char *uri_string,
-			    char *form_data)
-{
-	SoupMessage *msg;
-	SoupURI *uri;
-
-	uri = soup_uri_new (uri_string);
-	if (!uri)
-		return NULL;
-
-	if (!strcmp (method, "GET")) {
-		g_free (uri->query);
-		uri->query = form_data;
-
-		msg = soup_message_new_from_uri (method, uri);
-	} else if (!strcmp (method, "POST") || !strcmp (method, "PUT")) {
-		msg = soup_message_new_from_uri (method, uri);
-
-		soup_message_set_request (
-			msg, SOUP_FORM_MIME_TYPE_URLENCODED,
-			SOUP_MEMORY_TAKE,
-			form_data, strlen (form_data));
-	} else {
-		g_warning ("invalid method passed to soup_form_request_new");
-		g_free (form_data);
-
-		/* Don't crash */
-		msg = soup_message_new_from_uri (method, uri);
-	}
-	soup_uri_free (uri);
-
-	return msg;
-}
-
-/**
- * soup_form_request_new:
- * @method: the HTTP method, either "GET" or "POST"
- * @uri: the URI to send the form data to
- * @first_field: name of the first form field
- * @...: value of @first_field, followed by additional field names
- * and values, terminated by %NULL.
- *
- * Creates a new %SoupMessage and sets it up to send the given data
- * to @uri via @method. (That is, if @method is "GET", it will encode
- * the form data into @uri's query field, and if @method is "POST", it
- * will encode it into the %SoupMessage's request_body.)
- *
- * Return value: (transfer full): the new %SoupMessage
- **/
-SoupMessage *
-soup_form_request_new (const char *method, const char *uri,
-		       const char  *first_field, ...)
-{
-	va_list args;
-	char *form_data;
-
-	va_start (args, first_field);
-	form_data = soup_form_encode_valist (first_field, args);
-	va_end (args);
-
-	return soup_form_request_for_data (method, uri, form_data);
-}
-
-/**
- * soup_form_request_new_from_hash:
- * @method: the HTTP method, either "GET" or "POST"
- * @uri: the URI to send the form data to
- * @form_data_set: (element-type utf8 utf8): the data to send to @uri
- *
- * Creates a new %SoupMessage and sets it up to send @form_data_set to
- * @uri via @method, as with soup_form_request_new().
- *
- * Return value: (transfer full): the new %SoupMessage
- **/
-SoupMessage *
-soup_form_request_new_from_hash (const char *method, const char *uri,
-				 GHashTable *form_data_set)
-{
-	return soup_form_request_for_data (
-		method, uri, soup_form_encode_hash (form_data_set));
-}
-
-/**
- * soup_form_request_new_from_datalist:
- * @method: the HTTP method, either "GET" or "POST"
- * @uri: the URI to send the form data to
- * @form_data_set: the data to send to @uri
- *
- * Creates a new %SoupMessage and sets it up to send @form_data_set to
- * @uri via @method, as with soup_form_request_new().
- *
- * Return value: (transfer full): the new %SoupMessage
- **/
-SoupMessage *
-soup_form_request_new_from_datalist (const char *method, const char *uri,
-				     GData **form_data_set)
-{
-	return soup_form_request_for_data (
-		method, uri, soup_form_encode_datalist (form_data_set));
-}
-
-/**
- * soup_form_request_new_from_multipart:
- * @uri: the URI to send the form data to
- * @multipart: a "multipart/form-data" #SoupMultipart
- *
- * Creates a new %SoupMessage and sets it up to send @multipart to
- * @uri via POST.
- *
- * To send a <literal>"multipart/form-data"</literal> POST, first
- * create a #SoupMultipart, using %SOUP_FORM_MIME_TYPE_MULTIPART as
- * the MIME type. Then use soup_multipart_append_form_string() and
- * soup_multipart_append_form_file() to add the value of each form
- * control to the multipart. (These are just convenience methods, and
- * you can use soup_multipart_append_part() if you need greater
- * control over the part headers.) Finally, call
- * soup_form_request_new_from_multipart() to serialize the multipart
- * structure and create a #SoupMessage.
- *
- * Return value: (transfer full): the new %SoupMessage
- *
- * Since: 2.26
- **/
-SoupMessage *
-soup_form_request_new_from_multipart (const char *uri,
-				      SoupMultipart *multipart)
-{
-	SoupMessage *msg;
-
-	msg = soup_message_new ("POST", uri);
-	soup_multipart_to_message (multipart, msg->request_headers,
-				   msg->request_body);
-	return msg;
-}
