@@ -23,8 +23,9 @@
 #include "soup-content-processor.h"
 #include "content-sniffer/soup-content-sniffer-stream.h"
 #include "soup-filter-input-stream.h"
+#include "soup-logger-private.h"
 #include "soup-message-private.h"
-#include "soup-message-queue.h"
+#include "soup-message-queue-item.h"
 #include "soup-misc.h"
 #include "soup-uri-utils-private.h"
 
@@ -162,9 +163,10 @@ soup_message_setup_body_istream (GInputStream *body_stream,
 
 static void
 request_body_stream_wrote_data_cb (SoupMessage *msg,
-				   guint        count)
+                                   const void  *buffer,
+                                   guint        count)
 {
-	soup_message_wrote_body_data (msg, count);
+        soup_message_wrote_body_data (msg, count);
 }
 
 static void
@@ -315,6 +317,7 @@ io_write (SoupMessage *msg, gboolean blocking,
 {
 	SoupClientMessageIOData *client_io = soup_message_get_io_data (msg);
 	SoupMessageIOData *io = &client_io->base;
+	SoupSessionFeature *logger;
 	gssize nwrote;
 
 	if (io->async_error) {
@@ -365,6 +368,10 @@ io_write (SoupMessage *msg, gboolean blocking,
 								io->write_encoding,
 								io->write_length);
 		io->write_state = SOUP_MESSAGE_IO_STATE_BODY;
+		logger = soup_session_get_feature_for_message (client_io->item->session,
+		                                               SOUP_TYPE_LOGGER, msg);
+		if (logger)
+		        soup_logger_request_body_setup (SOUP_LOGGER (logger), msg);
 		break;
 
 	case SOUP_MESSAGE_IO_STATE_BODY:
@@ -392,7 +399,6 @@ io_write (SoupMessage *msg, gboolean blocking,
 				break;
 			} else {
 				io->async_wait = g_cancellable_new ();
-				g_main_context_push_thread_default (io->async_context);
 				g_output_stream_splice_async (io->body_ostream,
 							      soup_message_get_request_body_stream (msg),
 							      G_OUTPUT_STREAM_SPLICE_CLOSE_SOURCE,
@@ -400,7 +406,6 @@ io_write (SoupMessage *msg, gboolean blocking,
 							      cancellable,
 							      (GAsyncReadyCallback)request_body_stream_wrote_cb,
 							      g_object_ref (msg));
-				g_main_context_pop_thread_default (io->async_context);
 				return FALSE;
 			}
 		} else
@@ -415,12 +420,10 @@ io_write (SoupMessage *msg, gboolean blocking,
 				g_clear_object (&io->body_ostream);
 			} else {
 				io->async_wait = g_cancellable_new ();
-				g_main_context_push_thread_default (io->async_context);
 				g_output_stream_close_async (io->body_ostream,
 							     soup_client_message_io_data_get_priority (client_io),
 							     cancellable,
 							     closed_async, g_object_ref (msg));
-				g_main_context_pop_thread_default (io->async_context);
 			}
 		}
 
@@ -809,7 +812,7 @@ soup_message_io_run (SoupMessage *msg,
 								 NULL);
 		g_source_set_priority (io->io_source,
 				       soup_client_message_io_data_get_priority (client_io));
-		g_source_attach (io->io_source, io->async_context);
+		g_source_attach (io->io_source, g_main_context_get_thread_default ());
 	} else {
 		if (soup_message_get_io_data (msg) == client_io)
 			soup_message_io_finish (msg, error);
@@ -882,7 +885,7 @@ io_run_until_read_async (SoupMessage *msg,
 								 (SoupMessageIOSourceFunc)io_run_until_read_ready,
 								 task);
 		g_source_set_priority (io->io_source, g_task_get_priority (task));
-                g_source_attach (io->io_source, io->async_context);
+                g_source_attach (io->io_source, g_main_context_get_thread_default ());
                 return;
         }
 
@@ -975,12 +978,10 @@ soup_message_send_request (SoupMessageQueueItem      *item,
 	io->base.completion_cb = completion_cb;
 	io->base.completion_data = user_data;
 
-	io->item = item;
-	soup_message_queue_item_ref (item);
+	io->item = soup_message_queue_item_ref (item);
 	io->base.iostream = g_object_ref (soup_connection_get_iostream (io->item->conn));
 	io->base.istream = SOUP_FILTER_INPUT_STREAM (g_io_stream_get_input_stream (io->base.iostream));
 	io->base.ostream = g_io_stream_get_output_stream (io->base.iostream);
-	io->base.async_context = g_main_context_ref_thread_default ();
 
 	io->base.read_header_buf = g_byte_array_new ();
 	io->base.write_buf = g_string_new (NULL);
