@@ -34,14 +34,13 @@
  * you should not read directly from it, and the results are undefined
  * if you do.
  *
- * Since: 2.40
  **/
 
-static void soup_multipart_input_stream_pollable_init (GPollableInputStreamInterface *pollable_interface, gpointer interface_data);
-
-G_DEFINE_TYPE_WITH_CODE (SoupMultipartInputStream, soup_multipart_input_stream, G_TYPE_FILTER_INPUT_STREAM,
-			 G_IMPLEMENT_INTERFACE (G_TYPE_POLLABLE_INPUT_STREAM,
-						soup_multipart_input_stream_pollable_init))
+/**
+ * SoupMultipartInputStream:
+ *
+ * Class for handling streams of multipart messages.
+ */
 
 enum {
 	PROP_0,
@@ -49,7 +48,11 @@ enum {
 	PROP_MESSAGE,
 };
 
-struct _SoupMultipartInputStreamPrivate {
+struct _SoupMultipartInputStream {
+	GFilterInputStream parent_instance;
+};
+
+typedef struct {
 	SoupMessage	        *msg;
 
 	gboolean	         done_with_part;
@@ -63,15 +66,23 @@ struct _SoupMultipartInputStreamPrivate {
 	gsize		         boundary_size;
 
 	goffset		        remaining_bytes;
-};
+} SoupMultipartInputStreamPrivate;
+
+static void soup_multipart_input_stream_pollable_init (GPollableInputStreamInterface *pollable_interface, gpointer interface_data);
+
+G_DEFINE_TYPE_WITH_CODE (SoupMultipartInputStream, soup_multipart_input_stream, G_TYPE_FILTER_INPUT_STREAM,
+                         G_ADD_PRIVATE (SoupMultipartInputStream)
+			 G_IMPLEMENT_INTERFACE (G_TYPE_POLLABLE_INPUT_STREAM,
+						soup_multipart_input_stream_pollable_init))
 
 static void
 soup_multipart_input_stream_dispose (GObject *object)
 {
 	SoupMultipartInputStream *multipart = SOUP_MULTIPART_INPUT_STREAM (object);
+        SoupMultipartInputStreamPrivate *priv = soup_multipart_input_stream_get_instance_private (multipart);
 
-	g_clear_object (&multipart->priv->msg);
-	g_clear_object (&multipart->priv->base_stream);
+	g_clear_object (&priv->msg);
+	g_clear_object (&priv->base_stream);
 
 	G_OBJECT_CLASS (soup_multipart_input_stream_parent_class)->dispose (object);
 }
@@ -80,11 +91,12 @@ static void
 soup_multipart_input_stream_finalize (GObject *object)
 {
 	SoupMultipartInputStream *multipart = SOUP_MULTIPART_INPUT_STREAM (object);
+        SoupMultipartInputStreamPrivate *priv = soup_multipart_input_stream_get_instance_private (multipart);
 
-	g_free (multipart->priv->boundary);
+	g_free (priv->boundary);
 
-	if (multipart->priv->meta_buf)
-		g_clear_pointer (&multipart->priv->meta_buf, g_byte_array_unref);
+	if (priv->meta_buf)
+		g_clear_pointer (&priv->meta_buf, g_byte_array_unref);
 
 	G_OBJECT_CLASS (soup_multipart_input_stream_parent_class)->finalize (object);
 }
@@ -94,10 +106,11 @@ soup_multipart_input_stream_set_property (GObject *object, guint prop_id,
 					  const GValue *value, GParamSpec *pspec)
 {
 	SoupMultipartInputStream *multipart = SOUP_MULTIPART_INPUT_STREAM (object);
+        SoupMultipartInputStreamPrivate *priv = soup_multipart_input_stream_get_instance_private (multipart);
 
 	switch (prop_id) {
 	case PROP_MESSAGE:
-		multipart->priv->msg = g_value_dup_object (value);
+		priv->msg = g_value_dup_object (value);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -110,10 +123,11 @@ soup_multipart_input_stream_get_property (GObject *object, guint prop_id,
 					  GValue *value, GParamSpec *pspec)
 {
 	SoupMultipartInputStream *multipart = SOUP_MULTIPART_INPUT_STREAM (object);
+        SoupMultipartInputStreamPrivate *priv = soup_multipart_input_stream_get_instance_private (multipart);
 
 	switch (prop_id) {
 	case PROP_MESSAGE:
-		g_value_set_object (value, multipart->priv->msg);
+		g_value_set_object (value, priv->msg);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -130,7 +144,7 @@ soup_multipart_input_stream_read_real (GInputStream	*stream,
 				       GError          **error)
 {
 	SoupMultipartInputStream *multipart = SOUP_MULTIPART_INPUT_STREAM (stream);
-	SoupMultipartInputStreamPrivate *priv = multipart->priv;
+        SoupMultipartInputStreamPrivate *priv = soup_multipart_input_stream_get_instance_private (multipart);
 	gboolean got_boundary = FALSE;
 	gssize nread = 0;
 	guint8 *buf;
@@ -200,10 +214,7 @@ soup_multipart_input_stream_read (GInputStream	*stream,
 static void
 soup_multipart_input_stream_init (SoupMultipartInputStream *multipart)
 {
-	SoupMultipartInputStreamPrivate *priv;
-	priv = multipart->priv = G_TYPE_INSTANCE_GET_PRIVATE (multipart,
-							      SOUP_TYPE_MULTIPART_INPUT_STREAM,
-							      SoupMultipartInputStreamPrivate);
+	SoupMultipartInputStreamPrivate *priv = soup_multipart_input_stream_get_instance_private (multipart);
 
 	priv->meta_buf = g_byte_array_sized_new (RESPONSE_BLOCK_SIZE);
 	priv->done_with_part = FALSE;
@@ -212,19 +223,16 @@ soup_multipart_input_stream_init (SoupMultipartInputStream *multipart)
 static void
 soup_multipart_input_stream_constructed (GObject *object)
 {
-	SoupMultipartInputStream *multipart;
-	SoupMultipartInputStreamPrivate *priv;
+	SoupMultipartInputStream *multipart = (SoupMultipartInputStream*)object;
+	SoupMultipartInputStreamPrivate *priv = soup_multipart_input_stream_get_instance_private (multipart);
 	GInputStream *base_stream;
 	const char* boundary;
 	GHashTable *params = NULL;
 
-	multipart = SOUP_MULTIPART_INPUT_STREAM (object);
-	priv = multipart->priv;
-
 	base_stream = G_FILTER_INPUT_STREAM (multipart)->base_stream;
 	priv->base_stream = SOUP_FILTER_INPUT_STREAM (soup_filter_input_stream_new (base_stream));
 
-	soup_message_headers_get_content_type (priv->msg->response_headers,
+	soup_message_headers_get_content_type (soup_message_get_response_headers (priv->msg),
 					       &params);
 
 	boundary = g_hash_table_lookup (params, "boundary");
@@ -249,7 +257,7 @@ static gboolean
 soup_multipart_input_stream_is_readable (GPollableInputStream *stream)
 {
 	SoupMultipartInputStream *multipart = SOUP_MULTIPART_INPUT_STREAM (stream);
-	SoupMultipartInputStreamPrivate *priv = multipart->priv;
+	SoupMultipartInputStreamPrivate *priv = soup_multipart_input_stream_get_instance_private (multipart);
 
 	return g_pollable_input_stream_is_readable (G_POLLABLE_INPUT_STREAM (priv->base_stream));
 }
@@ -272,7 +280,7 @@ soup_multipart_input_stream_create_source (GPollableInputStream *stream,
 					   GCancellable         *cancellable)
 {
 	SoupMultipartInputStream *multipart = SOUP_MULTIPART_INPUT_STREAM (stream);
-	SoupMultipartInputStreamPrivate *priv = multipart->priv;
+	SoupMultipartInputStreamPrivate *priv = soup_multipart_input_stream_get_instance_private (multipart);
 	GSource *base_source, *pollable_source;
 
 	base_source = g_pollable_input_stream_create_source (G_POLLABLE_INPUT_STREAM (priv->base_stream), cancellable);
@@ -290,8 +298,6 @@ soup_multipart_input_stream_class_init (SoupMultipartInputStreamClass *multipart
 	GInputStreamClass *input_stream_class =
 		G_INPUT_STREAM_CLASS (multipart_class);
 
-	g_type_class_add_private (multipart_class, sizeof (SoupMultipartInputStreamPrivate));
-
 	object_class->dispose = soup_multipart_input_stream_dispose;
 	object_class->finalize = soup_multipart_input_stream_finalize;
 	object_class->constructed = soup_multipart_input_stream_constructed;
@@ -306,7 +312,8 @@ soup_multipart_input_stream_class_init (SoupMultipartInputStreamClass *multipart
 				     "Message",
 				     "The SoupMessage",
 				     SOUP_TYPE_MESSAGE,
-				     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+				     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
+				     G_PARAM_STATIC_STRINGS));
 
 }
 
@@ -322,7 +329,7 @@ soup_multipart_input_stream_pollable_init (GPollableInputStreamInterface *pollab
 static void
 soup_multipart_input_stream_parse_headers (SoupMultipartInputStream *multipart)
 {
-	SoupMultipartInputStreamPrivate *priv = multipart->priv;
+	SoupMultipartInputStreamPrivate *priv = soup_multipart_input_stream_get_instance_private (multipart);
 	gboolean success;
 
 	priv->current_headers = soup_message_headers_new (SOUP_MESSAGE_HEADERS_MULTIPART);
@@ -348,7 +355,7 @@ soup_multipart_input_stream_read_headers (SoupMultipartInputStream  *multipart,
 					  GCancellable		    *cancellable,
 					  GError		   **error)
 {
-	SoupMultipartInputStreamPrivate *priv = multipart->priv;
+	SoupMultipartInputStreamPrivate *priv = soup_multipart_input_stream_get_instance_private (multipart);
 	guchar read_buf[RESPONSE_BLOCK_SIZE];
 	guchar *buf;
 	gboolean got_boundary = FALSE;
@@ -420,14 +427,13 @@ soup_multipart_input_stream_read_headers (SoupMultipartInputStream  *multipart,
  * @base_stream: the #GInputStream returned by sending the request.
  *
  * Creates a new #SoupMultipartInputStream that wraps the
- * #GInputStream obtained by sending the #SoupRequest. Reads should
+ * #GInputStream obtained by sending the #SoupMessage. Reads should
  * not be done directly through this object, use the input streams
  * returned by soup_multipart_input_stream_next_part() or its async
  * counterpart instead.
  *
  * Returns: a new #SoupMultipartInputStream
  *
- * Since: 2.40
  **/
 SoupMultipartInputStream *
 soup_multipart_input_stream_new (SoupMessage  *msg,
@@ -457,22 +463,23 @@ soup_multipart_input_stream_new (SoupMessage  *msg,
  * the part; a new call to this function should be done at that point,
  * to obtain the next part.
  *
- * Return value: (transfer full): a new #GInputStream, or %NULL if
- * there are no more parts
+ * Returns: (nullable) (transfer full): a new #GInputStream, or
+ * %NULL if there are no more parts
  *
- * Since: 2.40
  */
 GInputStream *
 soup_multipart_input_stream_next_part (SoupMultipartInputStream  *multipart,
 				       GCancellable	         *cancellable,
 				       GError                   **error)
 {
+        SoupMultipartInputStreamPrivate *priv = soup_multipart_input_stream_get_instance_private (multipart);
+
 	if (!soup_multipart_input_stream_read_headers (multipart, cancellable, error))
 		return NULL;
 
 	soup_multipart_input_stream_parse_headers (multipart);
 
-	multipart->priv->done_with_part = FALSE;
+	priv->done_with_part = FALSE;
 
 	return G_INPUT_STREAM (g_object_new (SOUP_TYPE_BODY_INPUT_STREAM,
 					     "base-stream", G_INPUT_STREAM (multipart),
@@ -514,7 +521,6 @@ soup_multipart_input_stream_next_part_thread (GTask        *task,
  * soup_multipart_input_stream_next_part() for details on the
  * workflow.
  *
- * Since: 2.40
  */
 void
 soup_multipart_input_stream_next_part_async (SoupMultipartInputStream *multipart,
@@ -546,14 +552,14 @@ soup_multipart_input_stream_next_part_async (SoupMultipartInputStream *multipart
  * soup_multipart_input_stream_next_part_finish:
  * @multipart: a #SoupMultipartInputStream.
  * @result: a #GAsyncResult.
- * @error: a #GError location to store any error, or NULL to ignore.
+ * @error: a #GError location to store any error, or %NULL to ignore.
  *
  * Finishes an asynchronous request for the next part.
  *
- * Return value: (transfer full): a newly created #GInputStream for
- * reading the next part or %NULL if there are no more parts.
+ * Returns: (nullable) (transfer full): a newly created
+ * #GInputStream for reading the next part or %NULL if there are no
+ * more parts.
  *
- * Since: 2.40
  */
 GInputStream *
 soup_multipart_input_stream_next_part_finish (SoupMultipartInputStream	*multipart,
@@ -579,14 +585,14 @@ soup_multipart_input_stream_next_part_finish (SoupMultipartInputStream	*multipar
  * Note that if a part had no headers at all an empty #SoupMessageHeaders
  * will be returned.
  *
- * Return value: (transfer none): a #SoupMessageHeaders containing the headers
- * for the part currently being processed or %NULL if the headers failed to
- * parse.
+ * Returns: (nullable) (transfer none): a #SoupMessageHeaders
+ * containing the headers for the part currently being processed or
+ * %NULL if the headers failed to parse.
  *
- * Since: 2.40
  */
 SoupMessageHeaders *
 soup_multipart_input_stream_get_headers (SoupMultipartInputStream *multipart)
 {
-	return multipart->priv->current_headers;
+        SoupMultipartInputStreamPrivate *priv = soup_multipart_input_stream_get_instance_private (multipart);
+	return priv->current_headers;
 }
