@@ -3,59 +3,58 @@
  * Copyright (C) 2008 Red Hat, Inc.
  */
 
-#ifndef __SOUP_MULTIPART_H__
-#define __SOUP_MULTIPART_H__ 1
+#pragma once
 
-#include <libsoup/soup-types.h>
-#include <libsoup/soup-message-body.h>
-#include <libsoup/soup-message-headers.h>
+#include "soup-types.h"
+#include "soup-message-body.h"
+#include "soup-message-headers.h"
 
 G_BEGIN_DECLS
 
-typedef struct SoupMultipart SoupMultipart;
+typedef struct _SoupMultipart SoupMultipart;
 
-SOUP_AVAILABLE_IN_2_26
+SOUP_AVAILABLE_IN_ALL
 GType soup_multipart_get_type (void);
 #define SOUP_TYPE_MULTIPART (soup_multipart_get_type ())
 
-SOUP_AVAILABLE_IN_2_26
+SOUP_AVAILABLE_IN_ALL
 SoupMultipart *soup_multipart_new              (const char          *mime_type);
-SOUP_AVAILABLE_IN_2_26
+SOUP_AVAILABLE_IN_ALL
 SoupMultipart *soup_multipart_new_from_message (SoupMessageHeaders  *headers,
-						SoupMessageBody     *body);
+						GBytes              *body);
 
-SOUP_AVAILABLE_IN_2_26
+SOUP_AVAILABLE_IN_ALL
 int      soup_multipart_get_length         (SoupMultipart       *multipart);
-SOUP_AVAILABLE_IN_2_26
+SOUP_AVAILABLE_IN_ALL
 gboolean soup_multipart_get_part           (SoupMultipart       *multipart,
 					    int                  part,
 					    SoupMessageHeaders **headers,
-					    SoupBuffer         **body);
+					    GBytes             **body);
 
-SOUP_AVAILABLE_IN_2_26
+SOUP_AVAILABLE_IN_ALL
 void     soup_multipart_append_part        (SoupMultipart       *multipart,
 					    SoupMessageHeaders  *headers,
-					    SoupBuffer          *body);
+					    GBytes              *body);
 
-SOUP_AVAILABLE_IN_2_26
+SOUP_AVAILABLE_IN_ALL
 void     soup_multipart_append_form_string (SoupMultipart       *multipart,
 					    const char          *control_name,
 					    const char          *data);
-SOUP_AVAILABLE_IN_2_26
+SOUP_AVAILABLE_IN_ALL
 void     soup_multipart_append_form_file   (SoupMultipart       *multipart,
 					    const char          *control_name,
 					    const char          *filename,
 					    const char          *content_type,
-					    SoupBuffer          *body);
+					    GBytes              *body);
 
-SOUP_AVAILABLE_IN_2_26
+SOUP_AVAILABLE_IN_ALL
 void     soup_multipart_to_message         (SoupMultipart       *multipart,
 					    SoupMessageHeaders  *dest_headers,
-					    SoupMessageBody     *dest_body);
+					    GBytes             **dest_body);
 
-SOUP_AVAILABLE_IN_2_26
+SOUP_AVAILABLE_IN_ALL
 void     soup_multipart_free               (SoupMultipart       *multipart);
 
-G_END_DECLS
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (SoupMultipart, soup_multipart_free)
 
-#endif /* __SOUP_MULTIPART_H__ */
+G_END_DECLS
