@@ -211,7 +211,7 @@ do_request (SoupSession        *session,
 	/* Cache writes are G_PRIORITY_LOW, so they won't have happened yet... */
 	soup_cache_flush ((SoupCache *)soup_session_get_feature (session, SOUP_TYPE_CACHE));
 
-	return nread ? g_memdup (buf, nread) : g_strdup ("");
+	return nread ? g_memdup2 (buf, nread) : g_strdup ("");
 }
 
 static void
@@ -597,10 +597,8 @@ do_refcounting_test (gconstpointer data)
 	g_object_unref (cache);
 	g_free (cache_dir);
 
-        g_test_skip ("FIXME: Session has 1 ref at the end, SessionAsync had a different clear_message function with different semantics.");
-
-        return;
-
+	while (g_main_context_pending (NULL))
+                g_main_context_iteration (NULL, FALSE);
 	soup_test_session_abort_unref (session);
 }
 
@@ -749,7 +747,7 @@ main (int argc, char **argv)
 
 	test_init (argc, argv, NULL);
 
-	server = soup_test_server_new (TRUE);
+	server = soup_test_server_new (SOUP_TEST_SERVER_IN_THREAD);
 	soup_server_add_handler (server, NULL, server_callback, NULL, NULL);
 	base_uri = soup_test_server_get_uri (server, "http", NULL);
 
