@@ -10,40 +10,76 @@
 
 #define MAX_POST_LENGTH (sizeof (SHORT_BODY))
 
-static SoupURI *base_uri;
+static GUri *base_uri;
 static GSList *events;
 
 static void
-event (SoupMessage *msg, const char *side, const char *message)
+client_event (SoupMessage *msg,
+	      const char  *message)
 {
-	char *data = g_strdup_printf ("%s-%s", side, message);
+	char *data = g_strdup_printf ("client-%s", message);
+
+	debug_printf (2, "  %s", data);
+	debug_printf (2, "\n");
+
+	events = g_slist_append (events, data);
+}
+
+static void
+server_event (SoupServerMessage *msg,
+	      const char        *message)
+{
+	char *data = g_strdup_printf ("server-%s", message);
 	gboolean record_status =
 		(!strcmp (data, "server-wrote_headers") ||
 		 !strcmp (data, "server-wrote_informational"));
+	const char *reason_phrase = soup_server_message_get_reason_phrase (msg);
+	guint status_code = soup_server_message_get_status (msg);
 
 	debug_printf (2, "  %s", data);
 	if (record_status)
-		debug_printf (2, " (%s)", msg->reason_phrase);
+		debug_printf (2, " (%s)", reason_phrase);
 	debug_printf (2, "\n");
 
 	events = g_slist_append (events, data);
 	if (record_status)
-		events = g_slist_append (events, GUINT_TO_POINTER (msg->status_code));
+		events = g_slist_append (events, GUINT_TO_POINTER (status_code));
 }
 
-#define EVENT_HANDLER(name)			\
+#define CLIENT_EVENT_HANDLER(name)		\
 static void					\
-name (SoupMessage *msg, gpointer side)		\
+client_##name (SoupMessage *msg, gpointer side)	\
 {						\
-	event (msg, side, #name);		\
+	client_event (msg, #name);		\
 }
 
-EVENT_HANDLER (got_informational)
-EVENT_HANDLER (got_headers)
-EVENT_HANDLER (got_body)
-EVENT_HANDLER (wrote_informational)
-EVENT_HANDLER (wrote_headers)
-EVENT_HANDLER (wrote_body)
+#define SERVER_EVENT_HANDLER(name)		\
+static void					\
+server_##name (SoupServerMessage *msg, gpointer side)	\
+{						\
+	server_event (msg, #name);		\
+}
+
+CLIENT_EVENT_HANDLER (got_informational)
+CLIENT_EVENT_HANDLER (got_headers)
+CLIENT_EVENT_HANDLER (got_body)
+CLIENT_EVENT_HANDLER (wrote_headers)
+CLIENT_EVENT_HANDLER (wrote_body)
+CLIENT_EVENT_HANDLER (finished)
+
+SERVER_EVENT_HANDLER (got_headers)
+SERVER_EVENT_HANDLER (got_body)
+SERVER_EVENT_HANDLER (wrote_informational)
+SERVER_EVENT_HANDLER (wrote_headers)
+SERVER_EVENT_HANDLER (wrote_body)
+SERVER_EVENT_HANDLER (finished)
+
+static void
+restarted (SoupMessage *msg,
+           GBytes      *body)
+{
+        soup_message_set_request_body_from_bytes (msg, "text/plain", body);
+}
 
 static void
 do_message (const char *path, gboolean long_body,
@@ -53,47 +89,55 @@ do_message (const char *path, gboolean long_body,
 	SoupSession *session;
 	SoupMessage *msg;
 	const char *body;
-	SoupURI *uri;
+	GUri *uri, *msg_uri;
 	va_list ap;
 	const char *expected_event;
 	char *actual_event;
 	int expected_status, actual_status;
+	GBytes *request_body;
+	GBytes *response_body;
 
-	uri = soup_uri_copy (base_uri);
-	if (auth) {
-		soup_uri_set_user (uri, "user");
-		soup_uri_set_password (uri, "pass");
-	}
-	soup_uri_set_path (uri, path);
-	msg = soup_message_new_from_uri ("POST", uri);
-	soup_uri_free (uri);
+	if (auth)
+                uri = soup_uri_copy (base_uri, SOUP_URI_USER, "user", SOUP_URI_PASSWORD, "pass", SOUP_URI_NONE);
+        else
+                uri = g_uri_ref (base_uri);
+
+        msg_uri = g_uri_parse_relative (uri, path, SOUP_HTTP_URI_FLAGS, NULL);
+	msg = soup_message_new_from_uri ("POST", msg_uri);
+	g_uri_unref (uri);
+	g_uri_unref (msg_uri);
 
 	body = long_body ? LONG_BODY : SHORT_BODY;
-	soup_message_set_request (msg, "text/plain", SOUP_MEMORY_STATIC,
-				  body, strlen (body));
-	soup_message_headers_append (msg->request_headers, "Connection", "close");
+	request_body = g_bytes_new_static (body, strlen (body));
+	soup_message_set_request_body_from_bytes (msg, "text/plain", request_body);
+	soup_message_headers_append (soup_message_get_request_headers (msg), "Connection", "close");
 	if (expect_continue) {
-		soup_message_headers_set_expectations (msg->request_headers,
+		soup_message_headers_set_expectations (soup_message_get_request_headers (msg),
 						       SOUP_EXPECTATION_CONTINUE);
 	}
 
-	g_signal_connect (msg, "got_informational",
-			  G_CALLBACK (got_informational), "client");
-	g_signal_connect (msg, "got_headers",
-			  G_CALLBACK (got_headers), "client");
-	g_signal_connect (msg, "got_body",
-			  G_CALLBACK (got_body), "client");
-	g_signal_connect (msg, "wrote_informational",
-			  G_CALLBACK (wrote_informational), "client");
-	g_signal_connect (msg, "wrote_headers",
-			  G_CALLBACK (wrote_headers), "client");
-	g_signal_connect (msg, "wrote_body",
-			  G_CALLBACK (wrote_body), "client");
+	g_signal_connect (msg, "got-informational",
+			  G_CALLBACK (client_got_informational), NULL);
+	g_signal_connect (msg, "got-headers",
+			  G_CALLBACK (client_got_headers), NULL);
+	g_signal_connect (msg, "got-body",
+			  G_CALLBACK (client_got_body), NULL);
+	g_signal_connect (msg, "wrote-headers",
+			  G_CALLBACK (client_wrote_headers), NULL);
+	g_signal_connect (msg, "wrote-body",
+			  G_CALLBACK (client_wrote_body), NULL);
+	g_signal_connect (msg, "finished",
+			  G_CALLBACK (client_finished), NULL);
+	g_signal_connect (msg, "restarted",
+			  G_CALLBACK (restarted), request_body);
 
 	events = NULL;
-	session = soup_test_session_new (SOUP_TYPE_SESSION_ASYNC, NULL);
-	soup_session_send_message (session, msg);
+	session = soup_test_session_new (NULL);
+        g_assert (SOUP_IS_MESSAGE (msg));
+	response_body = soup_test_session_async_send (session, msg, NULL, NULL);
+        g_assert (SOUP_IS_MESSAGE (msg));
 	soup_test_session_abort_unref (session);
+        g_assert (SOUP_IS_MESSAGE (msg));
 
 	va_start (ap, auth);
 	while ((expected_event = va_arg (ap, const char *))) {
@@ -142,6 +186,8 @@ do_message (const char *path, gboolean long_body,
 		    !strcmp (actual_event, "server-wrote_informational"))
 			events = g_slist_delete_link (events, events);
 	}
+	g_bytes_unref (request_body);
+	g_bytes_unref (response_body);
 	g_object_unref (msg);
 }
 
@@ -150,13 +196,15 @@ do_test_unauth_short_noexpect_nopass (void)
 {
 	do_message ("/unauth", FALSE, FALSE, FALSE,
 		    "client-wrote_headers",
-		    "client-wrote_body",
 		    "server-got_headers",
 		    "server-got_body",
 		    "server-wrote_headers", SOUP_STATUS_CREATED,
 		    "server-wrote_body",
+		    "server-finished",
+		    "client-wrote_body",
 		    "client-got_headers",
 		    "client-got_body",
+		    "client-finished",
 		    NULL);
 }
 
@@ -165,13 +213,15 @@ do_test_unauth_long_noexpect_nopass (void)
 {
 	do_message ("/unauth", TRUE, FALSE, FALSE,
 		    "client-wrote_headers",
-		    "client-wrote_body",
 		    "server-got_headers",
 		    "server-got_body",
 		    "server-wrote_headers", SOUP_STATUS_REQUEST_ENTITY_TOO_LARGE,
 		    "server-wrote_body",
+		    "server-finished",
+		    "client-wrote_body",
 		    "client-got_headers",
 		    "client-got_body",
+		    "client-finished",
 		    NULL);
 }
 
@@ -183,12 +233,14 @@ do_test_unauth_short_expect_nopass (void)
 		    "server-got_headers",
 		    "server-wrote_informational", SOUP_STATUS_CONTINUE,
 		    "client-got_informational",
-		    "client-wrote_body",
 		    "server-got_body",
 		    "server-wrote_headers", SOUP_STATUS_CREATED,
 		    "server-wrote_body",
+		    "server-finished",
+		    "client-wrote_body",
 		    "client-got_headers",
 		    "client-got_body",
+		    "client-finished",
 		    NULL);
 }
 
@@ -200,8 +252,10 @@ do_test_unauth_long_expect_nopass (void)
 		    "server-got_headers",
 		    "server-wrote_headers", SOUP_STATUS_REQUEST_ENTITY_TOO_LARGE,
 		    "server-wrote_body",
+		    "server-finished",
 		    "client-got_headers",
 		    "client-got_body",
+		    "client-finished",
 		    NULL);
 }
 
@@ -210,13 +264,15 @@ do_test_auth_short_noexpect_nopass (void)
 {
 	do_message ("/auth", FALSE, FALSE, FALSE,
 		    "client-wrote_headers",
-		    "client-wrote_body",
 		    "server-got_headers",
 		    "server-got_body",
 		    "server-wrote_headers", SOUP_STATUS_UNAUTHORIZED,
 		    "server-wrote_body",
+		    "server-finished",
+		    "client-wrote_body",
 		    "client-got_headers",
 		    "client-got_body",
+		    "client-finished",
 		    NULL);
 }
 
@@ -225,13 +281,15 @@ do_test_auth_long_noexpect_nopass (void)
 {
 	do_message ("/auth", TRUE, FALSE, FALSE,
 		    "client-wrote_headers",
-		    "client-wrote_body",
 		    "server-got_headers",
 		    "server-got_body",
 		    "server-wrote_headers", SOUP_STATUS_UNAUTHORIZED,
 		    "server-wrote_body",
+		    "server-finished",
+		    "client-wrote_body",
 		    "client-got_headers",
 		    "client-got_body",
+		    "client-finished",
 		    NULL);
 }
 
@@ -243,8 +301,10 @@ do_test_auth_short_expect_nopass (void)
 		    "server-got_headers",
 		    "server-wrote_headers", SOUP_STATUS_UNAUTHORIZED,
 		    "server-wrote_body",
+		    "server-finished",
 		    "client-got_headers",
 		    "client-got_body",
+		    "client-finished",
 		    NULL);
 }
 
@@ -256,8 +316,10 @@ do_test_auth_long_expect_nopass (void)
 		    "server-got_headers",
 		    "server-wrote_headers", SOUP_STATUS_UNAUTHORIZED,
 		    "server-wrote_body",
+		    "server-finished",
 		    "client-got_headers",
 		    "client-got_body",
+		    "client-finished",
 		    NULL);
 }
 
@@ -266,21 +328,24 @@ do_test_auth_short_noexpect_pass (void)
 {
 	do_message ("/auth", FALSE, FALSE, TRUE,
 		    "client-wrote_headers",
-		    "client-wrote_body",
 		    "server-got_headers",
 		    "server-got_body",
 		    "server-wrote_headers", SOUP_STATUS_UNAUTHORIZED,
 		    "server-wrote_body",
+		    "server-finished",
+		    "client-wrote_body",
 		    "client-got_headers",
 		    "client-got_body",
 		    "client-wrote_headers",
-		    "client-wrote_body",
 		    "server-got_headers",
 		    "server-got_body",
 		    "server-wrote_headers", SOUP_STATUS_CREATED,
 		    "server-wrote_body",
+		    "server-finished",
+		    "client-wrote_body",
 		    "client-got_headers",
 		    "client-got_body",
+		    "client-finished",
 		    NULL);
 }
 
@@ -289,21 +354,24 @@ do_test_auth_long_noexpect_pass (void)
 {
 	do_message ("/auth", TRUE, FALSE, TRUE,
 		    "client-wrote_headers",
-		    "client-wrote_body",
 		    "server-got_headers",
 		    "server-got_body",
 		    "server-wrote_headers", SOUP_STATUS_UNAUTHORIZED,
 		    "server-wrote_body",
+		    "server-finished",
+		    "client-wrote_body",
 		    "client-got_headers",
 		    "client-got_body",
 		    "client-wrote_headers",
-		    "client-wrote_body",
 		    "server-got_headers",
 		    "server-got_body",
 		    "server-wrote_headers", SOUP_STATUS_REQUEST_ENTITY_TOO_LARGE,
 		    "server-wrote_body",
+		    "server-finished",
+		    "client-wrote_body",
 		    "client-got_headers",
 		    "client-got_body",
+		    "client-finished",
 		    NULL);
 }
 
@@ -315,18 +383,21 @@ do_test_auth_short_expect_pass (void)
 		    "server-got_headers",
 		    "server-wrote_headers", SOUP_STATUS_UNAUTHORIZED,
 		    "server-wrote_body",
+		    "server-finished",
 		    "client-got_headers",
 		    "client-got_body",
 		    "client-wrote_headers",
 		    "server-got_headers",
 		    "server-wrote_informational", SOUP_STATUS_CONTINUE,
 		    "client-got_informational",
-		    "client-wrote_body",
 		    "server-got_body",
 		    "server-wrote_headers", SOUP_STATUS_CREATED,
 		    "server-wrote_body",
+		    "server-finished",
+		    "client-wrote_body",
 		    "client-got_headers",
 		    "client-got_body",
+		    "client-finished",
 		    NULL);
 }
 
@@ -338,14 +409,17 @@ do_test_auth_long_expect_pass (void)
 		    "server-got_headers",
 		    "server-wrote_headers", SOUP_STATUS_UNAUTHORIZED,
 		    "server-wrote_body",
+		    "server-finished",
 		    "client-got_headers",
 		    "client-got_body",
 		    "client-wrote_headers",
 		    "server-got_headers",
 		    "server-wrote_headers", SOUP_STATUS_REQUEST_ENTITY_TOO_LARGE,
 		    "server-wrote_body",
+		    "server-finished",
 		    "client-got_headers",
 		    "client-got_body",
+		    "client-finished",
 		    NULL);
 }
 
@@ -353,45 +427,54 @@ do_test_auth_long_expect_pass (void)
 /* SERVER */
 
 static void
-server_got_headers (SoupMessage *msg, gpointer server)
+_server_got_headers (SoupServerMessage *msg,
+		     gpointer           server)
 {
+	guint status_code;
+	SoupMessageHeaders *request_headers;
+
+	status_code = soup_server_message_get_status (msg);
 	/* FIXME */
-	if (msg->status_code != SOUP_STATUS_CONTINUE &&
-	    msg->status_code != 0)
+	if (status_code != SOUP_STATUS_CONTINUE && status_code != 0)
 		return;
 
-	if (soup_message_headers_get_expectations (msg->request_headers) &
+	request_headers = soup_server_message_get_request_headers (msg);
+	if (soup_message_headers_get_expectations (request_headers) &
 	    SOUP_EXPECTATION_CONTINUE) {
 		const char *length;
 
-		length = soup_message_headers_get_one (msg->request_headers,
+		length = soup_message_headers_get_one (request_headers,
 						       "Content-Length");
 		if (length && atoi (length) > MAX_POST_LENGTH) {
-			soup_message_set_status (msg, SOUP_STATUS_REQUEST_ENTITY_TOO_LARGE);
-			soup_message_headers_append (msg->response_headers, "Connection", "close");
+			SoupMessageHeaders *response_headers;
+
+			response_headers = soup_server_message_get_response_headers (msg);
+			soup_server_message_set_status (msg, SOUP_STATUS_REQUEST_ENTITY_TOO_LARGE, NULL);
+			soup_message_headers_append (response_headers, "Connection", "close");
 		}
 	}
-}	
+}
 
 static void
-request_started (SoupServer *server, SoupMessage *msg,
-		 SoupClientContext *client, gpointer user_data)
+request_started (SoupServer        *server,
+		 SoupServerMessage *msg,
+		 gpointer           user_data)
 {
-	g_signal_connect (msg, "got_headers",
-			  G_CALLBACK (server_got_headers), server);
+	g_signal_connect (msg, "got-headers",
+			  G_CALLBACK (_server_got_headers), server);
 
-	g_signal_connect (msg, "got_informational",
-			  G_CALLBACK (got_informational), "server");
-	g_signal_connect (msg, "got_headers",
-			  G_CALLBACK (got_headers), "server");
-	g_signal_connect (msg, "got_body",
-			  G_CALLBACK (got_body), "server");
-	g_signal_connect (msg, "wrote_informational",
-			  G_CALLBACK (wrote_informational), "server");
-	g_signal_connect (msg, "wrote_headers",
-			  G_CALLBACK (wrote_headers), "server");
-	g_signal_connect (msg, "wrote_body",
-			  G_CALLBACK (wrote_body), "server");
+	g_signal_connect (msg, "got-headers",
+			  G_CALLBACK (server_got_headers), NULL);
+	g_signal_connect (msg, "got-body",
+			  G_CALLBACK (server_got_body), NULL);
+	g_signal_connect (msg, "wrote-informational",
+			  G_CALLBACK (server_wrote_informational), NULL);
+	g_signal_connect (msg, "wrote-headers",
+			  G_CALLBACK (server_wrote_headers), NULL);
+	g_signal_connect (msg, "wrote-body",
+			  G_CALLBACK (server_wrote_body), NULL);
+	g_signal_connect (msg, "finished",
+			  G_CALLBACK (server_finished), NULL);
 }
 
 static gboolean
@@ -402,18 +485,25 @@ auth_callback (SoupAuthDomain *auth_domain, SoupMessage *msg,
 }
 
 static void
-server_callback (SoupServer *server, SoupMessage *msg,
-		 const char *path, GHashTable *query,
-		 SoupClientContext *context, gpointer data)
+server_callback (SoupServer        *server,
+		 SoupServerMessage *msg,
+		 const char        *path,
+		 GHashTable        *query,
+		 gpointer           data)
 {
-	if (msg->method != SOUP_METHOD_POST) {
-		soup_message_set_status (msg, SOUP_STATUS_NOT_IMPLEMENTED);
-		soup_message_headers_append (msg->response_headers, "Connection", "close");
-	} else if (msg->request_body->length > MAX_POST_LENGTH) {
-		soup_message_set_status (msg, SOUP_STATUS_REQUEST_ENTITY_TOO_LARGE);
-		soup_message_headers_append (msg->response_headers, "Connection", "close");
+	SoupMessageHeaders *response_headers;
+	SoupMessageBody *request_body;
+
+	response_headers = soup_server_message_get_response_headers (msg);
+	request_body = soup_server_message_get_request_body (msg);
+	if (soup_server_message_get_method (msg) != SOUP_METHOD_POST) {
+		soup_server_message_set_status (msg, SOUP_STATUS_NOT_IMPLEMENTED, NULL);
+		soup_message_headers_append (response_headers, "Connection", "close");
+	} else if (request_body->length > MAX_POST_LENGTH) {
+		soup_server_message_set_status (msg, SOUP_STATUS_REQUEST_ENTITY_TOO_LARGE, NULL);
+		soup_message_headers_append (response_headers, "Connection", "close");
 	} else
-		soup_message_set_status (msg, SOUP_STATUS_CREATED);
+		soup_server_message_set_status (msg, SOUP_STATUS_CREATED, NULL);
 }
 
 static SoupServer *
@@ -430,10 +520,10 @@ setup_server (void)
 	soup_server_add_handler (server, NULL, server_callback, NULL, NULL);
 
 	auth_domain = soup_auth_domain_basic_new (
-		SOUP_AUTH_DOMAIN_REALM, "continue-test",
-		SOUP_AUTH_DOMAIN_ADD_PATH, "/auth",
-		SOUP_AUTH_DOMAIN_BASIC_AUTH_CALLBACK, auth_callback,
+		"realm", "continue-test",
+		"auth-callback", auth_callback,
 		NULL);
+        soup_auth_domain_add_path (auth_domain, "/auth");
 	soup_server_add_auth_domain (server, auth_domain);
 	g_object_unref (auth_domain);
 
@@ -469,7 +559,7 @@ main (int argc, char **argv)
 	ret = g_test_run ();
 
 	soup_test_server_quit_unref (server);
-	soup_uri_free (base_uri);
+	g_uri_unref (base_uri);
 
 	test_cleanup ();
 
