@@ -1032,19 +1032,16 @@ do_param_list_tests (void)
 #define RFC5987_TEST_HEADER_UTF8     "attachment; filename*=UTF-8''t%C3%A9st.txt; filename=\"test.txt\""
 #define RFC5987_TEST_HEADER_ISO      "attachment; filename=\"test.txt\"; filename*=iso-8859-1''t%E9st.txt"
 #define RFC5987_TEST_HEADER_FALLBACK "attachment; filename*=Unknown''t%FF%FF%FFst.txt; filename=\"test.txt\""
-#define RFC5987_TEST_HEADER_NO_TYPE  "filename=\"test.txt\""
-#define RFC5987_TEST_HEADER_NO_TYPE_2  "filename=\"test.txt\"; foo=bar"
 
 static void
 do_content_disposition_tests (void)
 {
 	SoupMessageHeaders *hdrs;
 	GHashTable *params;
-	const char *header, *filename, *parameter2;
+	const char *header, *filename;
 	char *disposition;
-	SoupBuffer *buffer;
+	GBytes *buffer;
 	SoupMultipart *multipart;
-	SoupMessageBody *body;
 
 	hdrs = soup_message_headers_new (SOUP_MESSAGE_HEADERS_MULTIPART);
 	params = g_hash_table_new (g_str_hash, g_str_equal);
@@ -1103,60 +1100,24 @@ do_content_disposition_tests (void)
 	g_assert_cmpstr (filename, ==, RFC5987_TEST_FALLBACK_FILENAME);
 	g_hash_table_destroy (params);
 
-        /* Invalid disposition with only a filename still works */
-        soup_message_headers_clear (hdrs);
-        soup_message_headers_append (hdrs, "Content-Disposition",
-				     RFC5987_TEST_HEADER_NO_TYPE);
-	if (!soup_message_headers_get_content_disposition (hdrs,
-							   &disposition,
-							   &params)) {
-		soup_test_assert (FALSE, "filename-only decoding FAILED");
-		return;
-	}
-        g_assert_null (disposition);
-        filename = g_hash_table_lookup (params, "filename");
-	g_assert_cmpstr (filename, ==, RFC5987_TEST_FALLBACK_FILENAME);
-	g_hash_table_destroy (params);
-
-        /* Invalid disposition with only two parameters still works */
-        soup_message_headers_clear (hdrs);
-        soup_message_headers_append (hdrs, "Content-Disposition",
-				     RFC5987_TEST_HEADER_NO_TYPE_2);
-	if (!soup_message_headers_get_content_disposition (hdrs,
-							   &disposition,
-							   &params)) {
-		soup_test_assert (FALSE, "only two parameters decoding FAILED");
-		return;
-	}
-        g_assert_null (disposition);
-        filename = g_hash_table_lookup (params, "filename");
-	g_assert_cmpstr (filename, ==, RFC5987_TEST_FALLBACK_FILENAME);
-        parameter2 = g_hash_table_lookup (params, "foo");
-        g_assert_cmpstr (parameter2, ==, "bar");
-	g_hash_table_destroy (params);
-
 	soup_message_headers_free (hdrs);
 
 	/* Ensure that soup-multipart always quotes filename */
 	g_test_bug ("641280");
 	multipart = soup_multipart_new (SOUP_FORM_MIME_TYPE_MULTIPART);
-	buffer = soup_buffer_new (SOUP_MEMORY_STATIC, "foo", 3);
+	buffer = g_bytes_new_static ("foo", 3);
 	soup_multipart_append_form_file (multipart, "test", "token",
 					 "text/plain", buffer);
-	soup_buffer_free (buffer);
+	g_bytes_unref (buffer);
 
 	hdrs = soup_message_headers_new (SOUP_MESSAGE_HEADERS_MULTIPART);
-	body = soup_message_body_new ();
-	soup_multipart_to_message (multipart, hdrs, body);
+	soup_multipart_to_message (multipart, hdrs, &buffer);
 	soup_message_headers_free (hdrs);
 	soup_multipart_free (multipart);
 
-	buffer = soup_message_body_flatten (body);
-	soup_message_body_free (body);
+	g_assert_nonnull (strstr (g_bytes_get_data (buffer, NULL), "filename=\"token\""));
 
-	g_assert_true (strstr (buffer->data, "filename=\"token\""));
-
-	soup_buffer_free (buffer);
+	g_bytes_unref (buffer);
 }
 
 #define CONTENT_TYPE_TEST_MIME_TYPE "text/plain"
