@@ -89,7 +89,7 @@ do_test_for_session (SoupSession *session,
 	debug_printf (2, "    requesting timeout\n");
 	timeout_uri = g_uri_parse_relative (base_uri, "/request-timeout", SOUP_HTTP_URI_FLAGS, NULL);
 	msg = soup_message_new_from_uri ("GET", timeout_uri);
-	body = soup_test_session_send (session, msg, NULL, NULL);
+	body = soup_session_send_and_read (session, msg, NULL, NULL);
 	g_bytes_unref (body);
 	g_uri_unref (timeout_uri);
 
@@ -121,7 +121,7 @@ do_test_for_session (SoupSession *session,
 	msg = soup_message_new_from_uri ("GET", base_uri);
 	server_processed_message = local_timeout = FALSE;
 	timeout_id = g_idle_add_full (G_PRIORITY_HIGH, timeout_cb, &local_timeout, NULL);
-	body = soup_test_session_send (session, msg, NULL, NULL);
+	body = soup_session_send_and_read (session, msg, NULL, NULL);
         g_bytes_unref (body);
 	g_object_unref (msg);
 
@@ -341,170 +341,82 @@ do_features_test (void)
 	soup_test_session_abort_unref (session);
 }
 
-typedef enum {
-        SYNC = 1 << 0,
-        STREAM = 1 << 1
-} GetTestFlags;
-
-typedef struct {
-        GMainLoop *loop;
-        GBytes *body;
-        char *content_type;
-        GError *error;
-} GetAsyncData;
-
-static GBytes *
-stream_to_bytes (GInputStream *stream)
+static void
+queue_order_test_message_finished (SoupMessage *msg,
+				   guint       *finished_count)
 {
-        GOutputStream *ostream;
-        GBytes *bytes;
-
-        ostream = g_memory_output_stream_new (NULL, 0, g_realloc, g_free);
-        g_output_stream_splice (ostream,
-                                stream,
-                                G_OUTPUT_STREAM_SPLICE_CLOSE_SOURCE |
-                                G_OUTPUT_STREAM_SPLICE_CLOSE_TARGET,
-                                NULL, NULL);
-        bytes = g_memory_output_stream_steal_as_bytes (G_MEMORY_OUTPUT_STREAM (ostream));
-        g_object_unref (ostream);
-
-        return bytes;
+	(*finished_count)++;
 }
 
 static void
-read_uri_async_ready_cb (SoupSession  *session,
-                         GAsyncResult *result,
-                         GetAsyncData *data)
+queue_order_test_message_network_event (SoupMessage       *msg,
+					GSocketClientEvent event,
+					GIOStream         *connection,
+					SoupMessage      **queue)
 {
-        GInputStream *stream;
-        goffset content_length;
+	int i;
 
-        stream = soup_session_read_uri_finish (session, result,
-                                               &content_length,
-                                               &data->content_type,
-                                               &data->error);
-        if (stream) {
-                data->body = stream_to_bytes (stream);
-                if (content_length != -1)
-                        g_assert_cmpint (g_bytes_get_size (data->body), ==, content_length);
-                g_object_unref (stream);
-        }
+	if (event != G_SOCKET_CLIENT_RESOLVING)
+                return;
 
-        g_main_loop_quit (data->loop);
+	for (i = 0; i < 3; i++) {
+		if (queue[i] == NULL) {
+			queue[i] = msg;
+			return;
+		}
+	}
+	g_assert_not_reached ();
 }
 
 static void
-load_uri_bytes_async_ready_cb (SoupSession  *session,
-                               GAsyncResult *result,
-                               GetAsyncData *data)
+do_queue_order_test (void)
 {
-        data->body = soup_session_load_uri_bytes_finish (session, result,
-                                                         &data->content_type,
-                                                         &data->error);
-        g_main_loop_quit (data->loop);
-}
+	SoupSession *session;
+	SoupMessage *msg1, *msg2, *msg3;
+	guint finished_count = 0;
+	SoupMessage *queue[3] = { NULL, NULL, NULL };
 
-static void
-do_read_uri_test (gconstpointer data)
-{
-        GUri *uri;
-        char *uri_string;
-        SoupSession *session;
-        GBytes *body = NULL;
-        char *content_type = NULL;
-        GError *error = NULL;
-        GetTestFlags flags = GPOINTER_TO_UINT (data);
+	session = soup_test_session_new (NULL);
 
-        session = soup_test_session_new (NULL);
+	msg1 = soup_message_new_from_uri ("GET", base_uri);
+	g_signal_connect (msg1, "network-event",
+			  G_CALLBACK (queue_order_test_message_network_event),
+			  queue);
+	g_signal_connect (msg1, "finished",
+			  G_CALLBACK (queue_order_test_message_finished),
+			  &finished_count);
+	msg2 = soup_message_new_from_uri ("GET", base_uri);
+	g_signal_connect (msg2, "network-event",
+			  G_CALLBACK (queue_order_test_message_network_event),
+			  queue);
+	g_signal_connect (msg2, "finished",
+			  G_CALLBACK (queue_order_test_message_finished),
+			  &finished_count);
+	msg3 = soup_message_new_from_uri ("GET", base_uri);
+	g_signal_connect (msg3, "network-event",
+			  G_CALLBACK (queue_order_test_message_network_event),
+			  queue);
+	g_signal_connect (msg3, "finished",
+			  G_CALLBACK (queue_order_test_message_finished),
+			  &finished_count);
 
-        uri = g_uri_parse_relative (base_uri, "/index.txt", SOUP_HTTP_URI_FLAGS, NULL);
-        uri_string = g_uri_to_string (uri);
+	soup_session_send_async (session, msg1, G_PRIORITY_DEFAULT, NULL, NULL, NULL);
+	soup_session_send_async (session, msg2, G_PRIORITY_DEFAULT, NULL, NULL, NULL);
+	soup_session_send_async (session, msg3, G_PRIORITY_DEFAULT, NULL, NULL, NULL);
+	while (queue[2] == NULL)
+		g_main_context_iteration (NULL, TRUE);
 
-        if (flags & SYNC) {
-                if (flags & STREAM) {
-                        GInputStream *stream;
-                        goffset content_length = 0;
+	g_assert_true (queue[0] == msg1);
+	g_assert_true (queue[1] == msg2);
+	g_assert_true (queue[2] == msg3);
+	g_object_unref (msg1);
+	g_object_unref (msg2);
+	g_object_unref (msg3);
 
-                        stream = soup_session_read_uri (session, uri_string, NULL,
-                                                        &content_length,
-                                                        &content_type,
-                                                        &error);
-                        body = stream_to_bytes (stream);
-                        if (content_length != -1)
-                                g_assert_cmpint (g_bytes_get_size (body), ==, content_length);
-                        g_object_unref (stream);
-                } else {
-                        body = soup_session_load_uri_bytes (session, uri_string, NULL,
-                                                            &content_type, &error);
-                }
-        } else {
-                GetAsyncData data;
-                GMainContext *context;
+	while (finished_count != 3)
+		g_main_context_iteration (NULL, TRUE);
 
-                memset (&data, 0, sizeof (GetAsyncData));
-
-                context = g_main_context_get_thread_default ();
-                data.loop = g_main_loop_new (context, TRUE);
-                if (flags & STREAM) {
-                        soup_session_read_uri_async (session, uri_string, G_PRIORITY_DEFAULT, NULL,
-                                                     (GAsyncReadyCallback)read_uri_async_ready_cb,
-                                                     &data);
-                } else {
-                        soup_session_load_uri_bytes_async (session, uri_string, G_PRIORITY_DEFAULT, NULL,
-                                                           (GAsyncReadyCallback)load_uri_bytes_async_ready_cb,
-                                                           &data);
-                }
-                g_main_loop_run (data.loop);
-                while (g_main_context_pending (context))
-                        g_main_context_iteration (context, FALSE);
-                g_main_loop_unref (data.loop);
-
-                body = data.body;
-                content_type = data.content_type;
-                if (data.error)
-                        g_propagate_error (&error, data.error);
-        }
-
-        g_assert_no_error (error);
-        g_assert_nonnull (body);
-        g_assert_cmpstr (content_type, ==, "text/plain");
-        g_assert_cmpmem (g_bytes_get_data (body, NULL), g_bytes_get_size (body),
-                         g_bytes_get_data (index_bytes, NULL), g_bytes_get_size (index_bytes));
-
-        g_bytes_unref (body);
-        g_free (content_type);
-        g_free (uri_string);
-        g_uri_unref (uri);
-
-        soup_test_session_abort_unref (session);
-}
-
-static struct {
-        const char *uri;
-        int expected_error;
-} get_error_tests[] = {
-        { "./foo", SOUP_SESSION_ERROR_BAD_URI },
-        { "http:/localhost/", SOUP_SESSION_ERROR_BAD_URI },
-        { "foo://host/path", SOUP_SESSION_ERROR_UNSUPPORTED_URI_SCHEME }
-};
-
-static void
-do_load_uri_error_tests (void)
-{
-        SoupSession *session;
-        guint i;
-
-        session = soup_test_session_new (NULL);
-
-        for (i = 0; i < G_N_ELEMENTS (get_error_tests); i++) {
-                GError *error = NULL;
-
-                g_assert_null (soup_session_load_uri_bytes (session, get_error_tests[i].uri, NULL, NULL, &error));
-                g_assert_error (error, SOUP_SESSION_ERROR, get_error_tests[i].expected_error);
-                g_error_free (error);
-        }
-
-        soup_test_session_abort_unref (session);
+	soup_test_session_abort_unref (session);
 }
 
 int
@@ -515,7 +427,7 @@ main (int argc, char **argv)
 
 	test_init (argc, argv, NULL);
 
-	server = soup_test_server_new (TRUE);
+	server = soup_test_server_new (SOUP_TEST_SERVER_IN_THREAD);
 	soup_server_add_handler (server, NULL, server_handler, NULL, NULL);
 	base_uri = soup_test_server_get_uri (server, "http", NULL);
 	index_bytes = soup_test_get_index ();
@@ -525,19 +437,7 @@ main (int argc, char **argv)
 	g_test_add_func ("/session/priority", do_priority_tests);
 	g_test_add_func ("/session/property", do_property_tests);
 	g_test_add_func ("/session/features", do_features_test);
-	g_test_add_data_func ("/session/read-uri/async",
-			      GINT_TO_POINTER (STREAM),
-			      do_read_uri_test);
-	g_test_add_data_func ("/session/read-uri/sync",
-			      GINT_TO_POINTER (SYNC | STREAM),
-			      do_read_uri_test);
-	g_test_add_data_func ("/session/load-uri/async",
-			      GINT_TO_POINTER (0),
-			      do_read_uri_test);
-	g_test_add_data_func ("/session/load-uri/sync",
-			      GINT_TO_POINTER (SYNC),
-			      do_read_uri_test);
-	g_test_add_func ("/session/load-uri/errors", do_load_uri_error_tests);
+	g_test_add_func ("/session/queue-order", do_queue_order_test);
 
 	ret = g_test_run ();
 
