@@ -5,7 +5,7 @@
  */
 
 #include "test-utils.h"
-#include "libsoup/soup-socket-private.h"
+#include "libsoup/server/soup-socket.h"
 
 #include <fcntl.h>
 #include <gio/gnetworking.h>
@@ -14,36 +14,38 @@
 #include <io.h>
 #endif
 
+static void
+assert_host_equals (GInetSocketAddress *addr, const char *host)
+{
+        char *addr_host = g_inet_address_to_string (g_inet_socket_address_get_address (addr));
+        g_assert_cmpstr (addr_host, ==, host);
+        g_free (addr_host);
+}
+
 static void
 do_unconnected_socket_test (void)
 {
-	SoupAddress *localhost;
+	GInetSocketAddress *addr;
+        GSocketAddress *localhost;
 	SoupSocket *sock;
-	SoupSocket *client;
-	SoupAddress *addr;
+	GSocketClient *client;
+	GSocketConnectable *remote_connectable;
+	GSocketConnection *conn;
+	GSocket *client_socket;
 	guint res;
-	struct sockaddr_in in_localhost;
 
 	g_test_bug ("673083");
 
-	in_localhost.sin_family = AF_INET;
-	in_localhost.sin_port = 0;
-	in_localhost.sin_addr.s_addr = htonl (INADDR_LOOPBACK);
+        localhost = g_inet_socket_address_new_from_string ("127.0.0.1", 0);
 
-	localhost = soup_address_new_from_sockaddr (
-		(struct sockaddr *) &in_localhost, sizeof (in_localhost));
-	g_assert_true (localhost != NULL);
-	res = soup_address_resolve_sync (localhost, NULL);
-	g_assert_cmpuint (res, ==, SOUP_STATUS_OK);
-
-	sock = soup_socket_new (SOUP_SOCKET_LOCAL_ADDRESS, localhost,
+	sock = soup_socket_new ("local-address", localhost,
 				NULL);
 	g_assert_true (sock != NULL);
 
 	addr = soup_socket_get_local_address (sock);
 	g_assert_true (addr != NULL);
-	g_assert_cmpstr (soup_address_get_physical (addr), ==, "127.0.0.1");
-	g_assert_cmpuint (soup_address_get_port (addr), ==, 0);
+	assert_host_equals (addr, "127.0.0.1");
+	g_assert_cmpuint (g_inet_socket_address_get_port (addr), ==, 0);
 
 	/* fails with ENOTCONN */
 	g_test_expect_message ("libsoup", G_LOG_LEVEL_WARNING,
@@ -52,30 +54,33 @@ do_unconnected_socket_test (void)
 	g_test_assert_expected_messages ();
 	g_assert_null (addr);
 
-	res = soup_socket_listen (sock);
+	res = soup_socket_listen (sock, NULL);
 	g_assert_true (res);
 
 	addr = soup_socket_get_local_address (sock);
 	g_assert_true (addr != NULL);
-	g_assert_cmpstr (soup_address_get_physical (addr), ==, "127.0.0.1");
-	g_assert_cmpuint (soup_address_get_port (addr), >, 0);
+	assert_host_equals (addr, "127.0.0.1");
+	g_assert_cmpuint (g_inet_socket_address_get_port (addr), >, 0);
 
-	client = soup_socket_new (SOUP_SOCKET_REMOTE_ADDRESS,
-				  soup_socket_get_local_address (sock),
-				  NULL);
-	res = soup_socket_connect_sync (client, NULL);
-	g_assert_cmpuint (res, ==, SOUP_STATUS_OK);
-	addr = soup_socket_get_local_address (client);
+	client = g_socket_client_new ();
+	remote_connectable = G_SOCKET_CONNECTABLE (soup_socket_get_local_address (sock));
+	conn = g_socket_client_connect (client, remote_connectable, NULL, NULL);
+	g_assert_true (conn != NULL);
+	client_socket = g_socket_connection_get_socket (conn);
+	g_assert_true (client_socket != NULL);
+	addr = G_INET_SOCKET_ADDRESS (g_socket_get_local_address (client_socket, NULL));
 	g_assert_true (addr != NULL);
-	addr = soup_socket_get_remote_address (client);
+	g_object_unref (addr);
+	addr = G_INET_SOCKET_ADDRESS (g_socket_get_remote_address (client_socket, NULL));
 	g_assert_true (addr != NULL);
-	g_assert_cmpstr (soup_address_get_physical (addr), ==, "127.0.0.1");
-	g_assert_cmpuint (soup_address_get_port (addr), >, 0);
+	assert_host_equals (addr, "127.0.0.1");
+	g_assert_cmpuint (g_inet_socket_address_get_port (addr), >, 0);
+	g_object_unref (addr);
+	g_object_unref (conn);
 	g_object_unref (client);
 
-	client = soup_socket_new (SOUP_SOCKET_REMOTE_ADDRESS,
-				  soup_socket_get_local_address (sock),
-				  NULL);
+	client = g_socket_client_new ();
+	remote_connectable = G_SOCKET_CONNECTABLE (soup_socket_get_local_address (sock));
 	/* save it for later */
 
 	/* listening socket fails with ENOTCONN */
@@ -97,86 +102,18 @@ do_unconnected_socket_test (void)
 	g_test_assert_expected_messages ();
 	g_assert_null (addr);
 
-	/* has never been connected */
-	g_test_expect_message ("libsoup", G_LOG_LEVEL_WARNING,
-			       "*socket not connected*");
-	addr = soup_socket_get_local_address (client);
-	g_test_assert_expected_messages ();
-	g_assert_null (addr);
-
-	res = soup_socket_connect_sync (client, NULL);
-	g_assert_cmpuint (res, ==, SOUP_STATUS_CANT_CONNECT);
-
-	g_test_expect_message ("libsoup", G_LOG_LEVEL_WARNING,
-			       "*socket not connected*");
-	addr = soup_socket_get_local_address (client);
-	g_test_assert_expected_messages ();
-	g_assert_null (addr);
+	conn = g_socket_client_connect (client, remote_connectable, NULL, NULL);
+	g_assert_false (conn != NULL);
 
 	g_object_unref (localhost);
 	g_object_unref (client);
 	g_object_unref (sock);
 }
 
-static void
-do_socket_from_fd_client_test (void)
+static int
+socket_get_fd (SoupSocket *socket)
 {
-	SoupServer *server;
-	SoupURI *uri;
-	GSocket *gsock;
-	SoupSocket *sock;
-	SoupAddress *local, *remote;
-	GSocketAddress *gaddr;
-	gboolean is_server;
-	GError *error = NULL;
-
-	server = soup_test_server_new (SOUP_TEST_SERVER_DEFAULT);
-	uri = soup_test_server_get_uri (server, "http", "127.0.0.1");
-
-	gsock = g_socket_new (G_SOCKET_FAMILY_IPV4,
-			      G_SOCKET_TYPE_STREAM,
-			      G_SOCKET_PROTOCOL_DEFAULT,
-			      &error);
-	g_assert_no_error (error);
-
-	gaddr = g_inet_socket_address_new_from_string ("127.0.0.1", uri->port);
-	g_socket_connect (gsock, gaddr, NULL, &error);
-	g_object_unref (gaddr);
-	g_assert_no_error (error);
-	g_assert_true (g_socket_is_connected (gsock));
-
-	gaddr = g_socket_get_local_address (gsock, &error);
-	g_assert_no_error (error);
-
-	sock = g_initable_new (SOUP_TYPE_SOCKET, NULL, &error,
-			       SOUP_SOCKET_FD, g_socket_get_fd (gsock),
-			       NULL);
-	g_assert_no_error (error);
-	g_assert_nonnull (sock);
-
-	g_object_get (G_OBJECT (sock),
-		      SOUP_SOCKET_LOCAL_ADDRESS, &local,
-		      SOUP_SOCKET_REMOTE_ADDRESS, &remote,
-		      SOUP_SOCKET_IS_SERVER, &is_server,
-		      NULL);
-	g_assert_cmpint (soup_socket_get_fd (sock), ==, g_socket_get_fd (gsock));
-	g_assert_false (is_server);
-	g_assert_true (soup_socket_is_connected (sock));
-
-	g_assert_cmpstr (soup_address_get_physical (local), ==, "127.0.0.1");
-	g_assert_cmpint (soup_address_get_port (local), ==, g_inet_socket_address_get_port (G_INET_SOCKET_ADDRESS (gaddr)));
-	g_assert_cmpstr (soup_address_get_physical (remote), ==, "127.0.0.1");
-	g_assert_cmpint (soup_address_get_port (remote), ==, uri->port);
-
-	g_object_unref (local);
-	g_object_unref (remote);
-	g_object_unref (gaddr);
-
-	g_object_unref (sock);
-	g_object_unref (gsock);
-
-	soup_test_server_quit_unref (server);
-	soup_uri_free (uri);
+        return g_socket_get_fd (soup_socket_get_gsocket (socket));
 }
 
 static void
@@ -184,9 +121,8 @@ do_socket_from_fd_server_test (void)
 {
 	GSocket *gsock;
 	SoupSocket *sock;
-	SoupAddress *local;
+	GInetSocketAddress *local;
 	GSocketAddress *gaddr;
-	gboolean is_server;
 	GError *error = NULL;
 
 	gsock = g_socket_new (G_SOCKET_FAMILY_IPV4,
@@ -207,21 +143,19 @@ do_socket_from_fd_server_test (void)
 	g_assert_no_error (error);
 
 	sock = g_initable_new (SOUP_TYPE_SOCKET, NULL, &error,
-			       SOUP_SOCKET_GSOCKET, gsock,
+			       "gsocket", gsock,
 			       NULL);
 	g_assert_no_error (error);
 	g_assert_nonnull (sock);
 
 	g_object_get (G_OBJECT (sock),
-		      SOUP_SOCKET_LOCAL_ADDRESS, &local,
-		      SOUP_SOCKET_IS_SERVER, &is_server,
+		      "local-address", &local,
 		      NULL);
-	g_assert_cmpint (soup_socket_get_fd (sock), ==, g_socket_get_fd (gsock));
-	g_assert_true (is_server);
+	g_assert_cmpint (socket_get_fd (sock), ==, g_socket_get_fd (gsock));
 	g_assert_true (soup_socket_is_connected (sock));
 
-	g_assert_cmpstr (soup_address_get_physical (local), ==, "127.0.0.1");
-	g_assert_cmpint (soup_address_get_port (local), ==, g_inet_socket_address_get_port (G_INET_SOCKET_ADDRESS (gaddr)));
+	assert_host_equals (local, "127.0.0.1");
+	g_assert_cmpint (g_inet_socket_address_get_port (local), ==, g_inet_socket_address_get_port (G_INET_SOCKET_ADDRESS (gaddr)));
 	g_object_unref (local);
 	g_object_unref (gaddr);
 
@@ -238,24 +172,10 @@ do_socket_from_fd_bad_test (void)
 {
 	GSocket *gsock, *gsock2, *gsockcli;
 	SoupSocket *sock, *sock2;
-	SoupAddress *local, *remote;
+	GInetSocketAddress *local, *remote;
 	GSocketAddress *gaddr;
-	gboolean is_server;
-	int fd;
 	GError *error = NULL;
 
-	/* Importing a non-socket fd gives an error */
-	fd = open (g_test_get_filename (G_TEST_DIST, "test-cert.pem", NULL), O_RDONLY);
-	g_assert_cmpint (fd, !=, -1);
-
-	sock = g_initable_new (SOUP_TYPE_SOCKET, NULL, &error,
-			       SOUP_SOCKET_FD, fd,
-			       NULL);
-	g_assert_error (error, G_IO_ERROR, G_IO_ERROR_FAILED);
-	g_clear_error (&error);
-	g_assert_null (sock);
-	close (fd);
-
 	/* Importing an unconnected socket gives an error */
 	gsock = g_socket_new (G_SOCKET_FAMILY_IPV4,
 			      G_SOCKET_TYPE_STREAM,
@@ -265,16 +185,13 @@ do_socket_from_fd_bad_test (void)
 	g_assert_false (g_socket_is_connected (gsock));
 
 	sock = g_initable_new (SOUP_TYPE_SOCKET, NULL, &error,
-			       SOUP_SOCKET_FD, g_socket_get_fd (gsock),
+			       "gsocket", gsock,
 			       NULL);
 	g_assert_error (error, G_IO_ERROR, G_IO_ERROR_FAILED);
 	g_clear_error (&error);
 	g_assert_null (sock);
 	g_object_unref (gsock);
 
-	/* Importing a non-listening server-side socket works, but
-	 * gives the wrong answer for soup_socket_is_server().
-	 */
 	gsock = g_socket_new (G_SOCKET_FAMILY_IPV4,
 			      G_SOCKET_TYPE_STREAM,
 			      G_SOCKET_PROTOCOL_DEFAULT,
@@ -307,29 +224,26 @@ do_socket_from_fd_bad_test (void)
 	g_assert_nonnull (gsock2);
 
 	sock2 = g_initable_new (SOUP_TYPE_SOCKET, NULL, &error,
-				SOUP_SOCKET_GSOCKET, gsock2,
+				"gsocket", gsock2,
 				NULL);
 	g_assert_no_error (error);
 	g_assert_nonnull (sock2);
 
 	g_object_get (G_OBJECT (sock2),
-		      SOUP_SOCKET_LOCAL_ADDRESS, &local,
-		      SOUP_SOCKET_REMOTE_ADDRESS, &remote,
-		      SOUP_SOCKET_IS_SERVER, &is_server,
+		      "local-address", &local,
+		      "remote-address", &remote,
 		      NULL);
-	g_assert_cmpint (soup_socket_get_fd (sock2), ==, g_socket_get_fd (gsock2));
+	g_assert_cmpint (socket_get_fd (sock2), ==, g_socket_get_fd (gsock2));
 	g_assert_true (soup_socket_is_connected (sock2));
-	/* This is wrong, but can't be helped. */
-	g_assert_false (is_server);
 
-	g_assert_cmpstr (soup_address_get_physical (local), ==, "127.0.0.1");
-	g_assert_cmpint (soup_address_get_port (local), ==, g_inet_socket_address_get_port (G_INET_SOCKET_ADDRESS (gaddr)));
+	assert_host_equals (local, "127.0.0.1");
+	g_assert_cmpint (g_inet_socket_address_get_port (local), ==, g_inet_socket_address_get_port (G_INET_SOCKET_ADDRESS (gaddr)));
 	g_object_unref (gaddr);
 
 	gaddr = g_socket_get_local_address (gsockcli, &error);
 	g_assert_no_error (error);
-	g_assert_cmpstr (soup_address_get_physical (remote), ==, "127.0.0.1");
-	g_assert_cmpint (soup_address_get_port (remote), ==, g_inet_socket_address_get_port (G_INET_SOCKET_ADDRESS (gaddr)));
+	assert_host_equals (remote, "127.0.0.1");
+	g_assert_cmpint (g_inet_socket_address_get_port (remote), ==, g_inet_socket_address_get_port (G_INET_SOCKET_ADDRESS (gaddr)));
 	g_object_unref (gaddr);
 
 	g_object_unref (local);
@@ -350,7 +264,6 @@ main (int argc, char **argv)
 	test_init (argc, argv, NULL);
 
 	g_test_add_func ("/sockets/unconnected", do_unconnected_socket_test);
-	g_test_add_func ("/sockets/from-fd/client", do_socket_from_fd_client_test);
 	g_test_add_func ("/sockets/from-fd/server", do_socket_from_fd_server_test);
 	g_test_add_func ("/sockets/from-fd/bad", do_socket_from_fd_bad_test);
 
