@@ -4,6 +4,9 @@
 #include "soup-misc.h"
 
 #include <glib/gprintf.h>
+#ifdef G_OS_UNIX
+#include <gio/gunixsocketaddress.h>
+#endif
 
 #include <locale.h>
 #include <signal.h>
@@ -286,7 +289,7 @@ soup_test_session_new (const char *propname, ...)
 	}
 
 	if (http_debug_level && !logger) {
-		SoupLoggerLogLevel level = MIN ((SoupLoggerLogLevel)http_debug_level, SOUP_LOGGER_LOG_HEADERS);
+		SoupLoggerLogLevel level = MIN ((SoupLoggerLogLevel)http_debug_level, SOUP_LOGGER_LOG_BODY);
 
 		logger = soup_logger_new (level);
 	}
@@ -313,28 +316,12 @@ typedef struct {
 } SendAsyncData;
 
 static void
-send_async_ready_cb (SoupSession   *session,
-		     GAsyncResult  *result,
-		     SendAsyncData *data)
+send_and_read_async_ready_cb (SoupSession   *session,
+			      GAsyncResult  *result,
+			      SendAsyncData *data)
 {
-	GInputStream *istream;
-	GOutputStream *ostream;
-
 	data->done = TRUE;
-	istream = soup_session_send_finish (session, result, &data->error);
-	if (!istream)
-		return;
-
-	ostream = g_memory_output_stream_new (NULL, 0, g_realloc, g_free);
-	g_output_stream_splice (ostream,
-				istream,
-				G_OUTPUT_STREAM_SPLICE_CLOSE_SOURCE |
-				G_OUTPUT_STREAM_SPLICE_CLOSE_TARGET,
-				NULL,
-				&data->error);
-	data->body = g_memory_output_stream_steal_as_bytes (G_MEMORY_OUTPUT_STREAM (ostream));
-	g_object_unref (ostream);
-	g_object_unref (istream);
+	data->body = soup_session_send_and_read_finish (session, result, &data->error);
 }
 
 static void
@@ -358,8 +345,8 @@ soup_test_session_async_send (SoupSession  *session,
 	signal_id = g_signal_connect (msg, "finished",
                                      G_CALLBACK (on_message_finished), &message_finished);
 
-	soup_session_send_async (session, msg, G_PRIORITY_DEFAULT, cancellable,
-				 (GAsyncReadyCallback)send_async_ready_cb, &data);
+	soup_session_send_and_read_async (session, msg, G_PRIORITY_DEFAULT, cancellable,
+					  (GAsyncReadyCallback)send_and_read_async_ready_cb, &data);
 
 	while (!data.done || !message_finished)
 		g_main_context_iteration (async_context, TRUE);
@@ -386,16 +373,27 @@ soup_test_session_send_message (SoupSession *session,
 	return soup_message_get_status (msg);
 }
 
+const char *
+soup_test_server_get_unix_path (SoupServer *server)
+{
+	return g_object_get_data (G_OBJECT (server), "unix-socket-path");
+}
+
 static void
 server_listen (SoupServer *server)
 {
 	GError *error = NULL;
         SoupServerListenOptions options = 0;
+	GSocket *socket;
 
         if (g_getenv ("SOUP_TEST_NO_IPV6"))
                 options = SOUP_SERVER_LISTEN_IPV4_ONLY;
 
-	soup_server_listen_local (server, 0, options, &error);
+	socket = g_object_get_data (G_OBJECT (server), "listen-socket");
+	if (socket != NULL)
+		soup_server_listen_socket (server, socket, 0, &error);
+	else
+		soup_server_listen_local (server, 0, options, &error);
 	if (error) {
 		g_printerr ("Unable to create server: %s\n", error->message);
 		exit (1);
@@ -480,6 +478,44 @@ soup_test_server_new (SoupTestServerOptions options)
 
 	g_object_set_data (G_OBJECT (server), "options", GUINT_TO_POINTER (options));
 
+	if (options & SOUP_TEST_SERVER_UNIX_SOCKET) {
+#ifdef G_OS_UNIX
+		char *socket_dir, *socket_path;
+		GSocket *listen_socket;
+		GSocketAddress *listen_address;
+
+		socket_dir = g_dir_make_tmp ("unix-socket-test-XXXXXX", NULL);
+		socket_path = g_build_filename (socket_dir, "socket", NULL);
+		g_object_set_data_full (G_OBJECT (server), "unix-socket-path", socket_path, g_free);
+		g_free (socket_dir);
+
+		listen_socket = g_socket_new (G_SOCKET_FAMILY_UNIX,
+					      G_SOCKET_TYPE_STREAM,
+					      G_SOCKET_PROTOCOL_DEFAULT,
+					      &error);
+		if (listen_socket == NULL) {
+			g_printerr ("Unable to create unix socket: %s\n", error->message);
+			exit (1);
+		}
+
+		listen_address = g_unix_socket_address_new (socket_path);
+		if (!g_socket_bind (listen_socket, listen_address, TRUE, &error)) {
+			g_printerr ("Unable to bind unix socket to %s: %s\n", socket_path, error->message);
+			exit (1);
+		}
+		g_object_unref (listen_address);
+		if (!g_socket_listen (listen_socket, &error)) {
+			g_printerr ("Unable to listen on unix socket: %s\n", error->message);
+			exit (1);
+		}
+
+		g_object_set_data_full (G_OBJECT (server), "listen-socket", listen_socket, g_object_unref);
+#else
+		g_printerr ("Unix socket support not available\n");
+		exit (1);
+#endif
+	}
+
 	if (options & SOUP_TEST_SERVER_IN_THREAD)
 		soup_test_server_run_in_thread (server);
 	else if (!(options & SOUP_TEST_SERVER_NO_DEFAULT_LISTENER))
@@ -708,6 +744,8 @@ soup_test_request_send (SoupSession   *session,
 				 async_as_sync_callback, &data);
 	g_main_loop_run (data.loop);
 
+	g_assert_true (soup_session_get_async_result_message (session, data.result) == msg);
+
 	stream = soup_session_send_finish (session, data.result, error);
 
 	if (flags & SOUP_TEST_REQUEST_CANCEL_AFTER_SEND_FINISH) {
@@ -773,33 +811,6 @@ soup_test_request_close_stream (GInputStream  *stream,
 	return ok;
 }
 
-GBytes *
-soup_test_session_send (SoupSession   *session,
-			SoupMessage   *msg,
-			GCancellable  *cancellable,
-			GError       **error)
-{
-	GInputStream *istream;
-	GOutputStream *ostream;
-	GBytes *body;
-
-	istream = soup_session_send (session, msg, cancellable, error);
-	if (!istream)
-		return NULL;
-
-	ostream = g_memory_output_stream_new (NULL, 0, g_realloc, g_free);
-	g_output_stream_splice (ostream,
-				istream,
-				G_OUTPUT_STREAM_SPLICE_CLOSE_SOURCE |
-				G_OUTPUT_STREAM_SPLICE_CLOSE_TARGET,
-				NULL, NULL);
-	body = g_memory_output_stream_steal_as_bytes (G_MEMORY_OUTPUT_STREAM (ostream));
-	g_object_unref (ostream);
-	g_object_unref (istream);
-
-	return body;
-}
-
 void
 soup_test_register_resources (void)
 {
